
/*
 * various hogs and tests
 *
 * Copyright (C) 2004 Herbert Poetzl <herbert@13thfloor.at>
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * V0.1 do some cpu hogging ...
 * V0.2 generalized cpuhog
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tests.h"
#include "timing.h"


int 	do_cpuhog(int loops, int msec)
{
	unsigned long long v1,v2,v3;
	int i;

	v1 = 42;
	v2 = v3 = -1;
	for (i=0; i<loops; i++) {
	    v1 += i;
	    v2 = v1*i + v3;
	    v3 = v1 ^ v2;
	}
	if (msec)
	    msleep(msec);
	return 0;
}

int 	do_syshog(int size, int loops, int msec)
{
	static void *buffer = NULL;
	int i, fd;

    	if (!buffer)
	buffer = realloc(buffer, size);

    	fd = open("/dev/zero", O_RDWR);
	if (fd>=0) {
	    for (i=0; i<loops; i++) {
 	    	read(fd, buffer, size);
    	    	write(fd, buffer, size);
	    }
	    close(fd);
	}
	if (msec)
	    msleep(msec);
	return 0;
}



    /* tests below */


int 	test_cpuhog(per_thread *ptd, const char *arg)
{
  	int count = atol(arg);

    	CPRINTF(0, "hogging cpu [1000,0]%d\n", count);
	while (count--)
	    do_cpuhog(1000, 0);
	return 0;
}

int 	test_syshog(per_thread *ptd, const char *arg)
{
  	int count = atol(arg);

    	CPRINTF(0, "hogging system [^20,100,0]%d\n", count);
	while (count--)
	    do_syshog((1<<20), 100, 0);
	return 0;
}

