
/*
 * vserver stress tests
 *
 * Copyright (C) 2004 Herbert Poetzl <herbert@13thfloor.at>
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * V0.1 context sequencer
 *
 */

#include "sequence.h"

unsigned long xid_array[XARRAY_SIZE];


void 	seq_clear(int num, long *array)
{
    	if (array) {
	    int bit = num % LONG_BITS;
	    long *p = array + (num/LONG_BITS);
	    
	    *p &= ~(1L << bit);
	}
}

void 	seq_set(int num, long *array)
{
    	if (array) {
	    int bit = num % LONG_BITS;
	    long *p = array + (num/LONG_BITS);
	    
	    *p |= (1L << bit);
	}
}

int 	seq_avail(int num, long *array)
{
    	if (array) {
	    int bit = num % LONG_BITS;
	    long *p = array + (num/LONG_BITS);
	    
	    return (*p & (1L << bit)) == 0;
    	}
	return (0==0);
}



int 	seq_next(sequence *seq, long *array)
{
    	int val, first;
	
	first = seq->count;
	
	spin_lock(&seq->lock);
	do {
            val = seq->count++;
	} while (!seq_avail(val, array));
	seq_set(val, array);
	spin_unlock(&seq->lock);
	return val;
}

