
/*
 * Copyright (C) 2004 Herbert Poetzl <herbert@13thfloor.at>
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _TESTS_H_
#define _TESTS_H_

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>

#define STACK_SIZE  4096
#define MAX_THREAD  32
#define MAX_BUFFER  256

extern  int verb_lvl;

typedef
struct	_per_thread {
    	int index, cpu, cycle;
    	struct timeval tv_start, tv_cycle, now;
    	char buffer[MAX_BUFFER];
}   	per_thread;

#define PTD_SIZE  sizeof(per_thread)


static inline
int tprintf(per_thread *ptd, const char *fmt, ...)
{
    	va_list ap;
	int len;
	
	va_start(ap, fmt);
    	len = vsnprintf(ptd->buffer, MAX_BUFFER, fmt, ap);
	write(0, ptd->buffer, len);
	va_end(ap);
	return len;
}

long gnow(per_thread *ptd);
long rnow(per_thread *ptd, struct timeval *ref);


#define DSPLIT(d)   ((d)/1000), ((d)%1000)

#define _G_TM_FMT   "%6ld.%03ld "
#define _U_TM_FMT   "%6ld.%03ld "
#define _T_TM_FMT   "%6ld.%03ld^"
#define _C_TM_FMT   "%6ld.%03ld"

#define _G_TM_VAL   gnow(NULL)
#define _U_TM_VAL   gnow(ptd)
#define _T_TM_VAL   rnow(ptd, &ptd->tv_start)
#define _C_TM_VAL   rnow(ptd, &ptd->tv_cycle)

#define _G_ID_FMT   "[%c]"
#define _U_ID_FMT   "[%d]"
#define _T_ID_FMT   "[%d%d]"
#define _C_ID_FMT   "[%d%d,%d]"

#define _G_ID_ARG   '*'
#define _U_ID_ARG   ptd->cpu
#define _T_ID_ARG   ptd->cpu, ptd->index
#define _C_ID_ARG   ptd->cpu, ptd->index, ptd->cycle



#define XPRINTF(v,t,i,f,x...) do {			\
    	if (verb_lvl>(v)) {				\
	    long _delta = t##_TM_VAL;	    	    	\
	    tprintf(ptd, t##_TM_FMT	    	    	\
	    	" " i##_ID_FMT " " f,	    		\
		DSPLIT(_delta),     	    	    	\
		i##_ID_ARG, ##x);  	    	    	\
	}} while (0)


#define DPRINTF(v,f,x...) XPRINTF(v,_G,_G,f, ##x)
#define UPRINTF(v,f,x...) XPRINTF(v,_U,_U,f, ##x)
#define TPRINTF(v,f,x...) XPRINTF(v,_T,_T,f, ##x)
#define CPRINTF(v,f,x...) XPRINTF(v,_C,_C,f, ##x)



int 	test_cpuhog(per_thread *ptd, const char *arg);
int 	test_syshog(per_thread *ptd, const char *arg);


#else
#warning duplicate inclusion.
#endif
