
/*
 * Copyright (C) 2004 Herbert Poetzl <herbert@13thfloor.at>
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _TIMING_H_
#define _TIMING_H_

#include <sys/time.h>


#define STOMS(x)    ((x)*1000)
#define STOUS(x)    ((x)*1000*1000)
#define MTOUS(x)    ((x)*1000)
#define UTOMS(x)    ((x)/1000)

#define USTOS(x)    (((x)/1000)/1000)
#define USREM(x)    ((x)%(1000*1000))

#define msleep(x)   usleep(MTOUS(x));


static inline
long tm_calc_overhead(int count) 
{
	struct timeval start_time, end_time;
	int i, adjust;

    	/* try to calculate gettimeofday() overhead */
	gettimeofday(&start_time, NULL);
    	for (i=0; i<count; i++)
	    gettimeofday(&end_time, NULL);

        adjust = STOUS(end_time.tv_sec - start_time.tv_sec)
	    + (end_time.tv_usec - start_time.tv_usec);
	adjust /= count;
    	return adjust;
}

static inline
void tm_start_time(struct timeval *tv)
{
    	gettimeofday(tv, NULL);
}

static inline
long tm_delta_us(struct timeval *start, struct timeval *now)
{
	long delta;
	
 	delta = STOUS(now->tv_sec - start->tv_sec)
	    + (now->tv_usec - start->tv_usec);
    	return delta;
}

static inline
long tm_delta_ms(struct timeval *start, struct timeval *now)
{
	long delta;
	
 	delta = STOMS(now->tv_sec - start->tv_sec)
	    + UTOMS(now->tv_usec - start->tv_usec);
    	return delta;
}

static inline
long tm_delta_time(struct timeval *tv)
{
    	struct timeval now;	
	long delta;

    	gettimeofday(&now, NULL);
 	delta = tm_delta_us(tv, &now);

	*tv = now;
	return delta;
}


#else
#warning duplicate inclusion.
#endif
